@@ -23,10 +23,10 @@ _units = microsec_units+millisec_units+sec_units+min_units+hr_units+day_units
 _calendars = ['standard', 'gregorian', 'proleptic_gregorian',
               'noleap', 'julian', 'all_leap', '365_day', '366_day', '360_day']
 
-__version__ = '1.4'
+__version__ = '1.4.1'
 
 # Adapted from http://delete.me.uk/2005/03/iso8601.html
-ISO8601_REGEX = re.compile(r"(?P<year>[0-9]{1,4})(-(?P<month>[0-9]{1,2})(-(?P<day>[0-9]{1,2})"
+ISO8601_REGEX = re.compile(r"(?P<year>[+-]?[0-9]{1,4})(-(?P<month>[0-9]{1,2})(-(?P<day>[0-9]{1,2})"
                            r"(((?P<separator1>.)(?P<hour>[0-9]{1,2}):(?P<minute>[0-9]{1,2})(:(?P<second>[0-9]{1,2})(\.(?P<fraction>[0-9]+))?)?)?"
                            r"((?P<separator2>.?)(?P<timezone>Z|(([-+])([0-9]{1,2}):([0-9]{1,2}))))?)?)?)?"
                            )
@@ -664,6 +664,14 @@ units to datetime objects.
             raise ValueError(
                 "calendar must be one of %s, got '%s'" % (str(_calendars), calendar))
         units, tzoffset, self.origin = _dateparse(unit_string)
+        # real-world calendars limited to positive reference years.
+        if self.calendar in ['julian', 'standard', 'gregorian', 'proleptic_gregorian']:
+            if self.origin.year == 0:
+                msg='zero not allowed as a reference year, does not exist in Julian or Gregorian calendars'
+                raise ValueError(msg)
+            elif self.origin.year < 0:
+                msg='negative reference year in time units, must be >= 1'
+                raise ValueError(msg)
         self.tzoffset = tzoffset  # time zone offset in minutes
         self.units = units
         self.unit_string = unit_string
@@ -900,7 +908,8 @@ def _parse_date(datestring):
     #    groups["fraction"] = 0
     # else:
     #    groups["fraction"] = int(float("0.%s" % groups["fraction"]) * 1e6)
-    return int(groups["year"]), int(groups["month"]), int(groups["day"]),\
+    iyear = int(groups["year"])
+    return iyear, int(groups["month"]), int(groups["day"]),\
         int(groups["hour"]), int(groups["minute"]), int(groups["second"]),\
         tzoffset_mins
 
@@ -995,6 +1004,10 @@ def date2index(dates, nctime, calendar=None, select='exact'):
     an exact match cannot be found. C{nearest} will return the indices that
     correpond to the closest dates.
     """
+    try:
+        nctime.units
+    except AttributeError:
+        raise AttributeError("netcdf time variable is missing a 'units' attribute")
     # Setting the calendar.
     if calendar == None:
         calendar = getattr(nctime, 'calendar', 'standard')
@@ -1029,6 +1042,10 @@ def time2index(times, nctime, calendar=None, select='exact'):
     an exact match cannot be found. C{nearest} will return the indices that
     correpond to the closest times.
     """
+    try:
+        nctime.units
+    except AttributeError:
+        raise AttributeError("netcdf time variable is missing a 'units' attribute")
     # Setting the calendar.
     if calendar == None:
         calendar = getattr(nctime, 'calendar', 'standard')
