@@ -1,13 +1,14 @@
 from netcdftime import utime, JulianDayFromDate, DateFromJulianDay
 from netcdftime import datetime as datetimex
-from netCDF4 import Dataset, num2date, date2num, date2index
+from netCDF4 import Dataset, num2date, date2num, date2index, num2date
+import copy
 import numpy
 import random
 import sys
 import unittest
 import os
 import tempfile
-from datetime import datetime
+from datetime import datetime, timedelta
 from numpy.testing import assert_almost_equal, assert_equal
 
 # test netcdftime module for netCDF time <--> python datetime conversions.
@@ -207,7 +208,7 @@ class netcdftimeTestCase(unittest.TestCase):
         assert_equal(numpy.around(t), 3600)
         # test fix for issue 75 (seconds hit 60 at end of month,
         # day goes out of range).
-        t = 733498.999999
+        t = 733499.0
         d = num2date(t, units='days since 0001-01-01 00:00:00')
         dateformat =  '%Y-%m-%d %H:%M:%S'
         assert_equal(d.strftime(dateformat), '2009-04-01 00:00:00')
@@ -412,6 +413,24 @@ class netcdftimeTestCase(unittest.TestCase):
             self.assertEqual(d1, d2)
             self.assertEqual(num2date(d1, units, cap_cal),
                              num2date(d1, units, low_cal))
+        # issue 415
+        t = datetimex(2001, 12, 1, 2, 3, 4)
+        self.assertEqual(t, copy.deepcopy(t))
+
+        # issue 442
+        units = "days since 0000-01-01 00:00:00"
+        # this should fail (year zero not allowed with real-world calendars)
+        try:
+            date2num(datetime(1, 1, 1), units, calendar='standard')
+        except ValueError:
+            pass
+        # this should not fail (year zero allowed in 'fake' calendars)
+        t = date2num(datetime(1, 1, 1), units, calendar='360_day')
+        self.assertEqual(t, 360)
+        d = num2date(t, units, calendar='360_day')
+        self.assertEqual(d, datetimex(1,1,1))
+        d = num2date(0, units, calendar='360_day')
+        self.assertEqual(d, datetimex(0,1,1))
 
 class TestDate2index(unittest.TestCase):
 
@@ -466,6 +485,16 @@ class TestDate2index(unittest.TestCase):
         time2.units = 'days since 1901-01-01'
         self.first_timestamp = datetime(2000, 1, 1)
         time2[0] = date2num(self.first_timestamp, time2.units)
+        ntimes = 21
+        f.createDimension("record", ntimes)
+        time3 = f.createVariable("time3", numpy.int32, ("record", ))
+        time3.units = "seconds since 1970-01-01 00:00:00"
+        date = datetime(2037,1,1,0)
+        dates = [date]
+        for ndate in range(ntimes-1):
+            date += (ndate+1)*timedelta(hours=1)
+            dates.append(date)
+        time3[:] = date2num(dates,time3.units)
         f.close()
 
     def tearDown(self):
@@ -587,5 +616,16 @@ class TestDate2index(unittest.TestCase):
             assert_equal(date, date2)
         f.close()
 
+    def test_issue444(self):
+        # make sure integer overflow not causing error in
+        # calculation of nearest index when sum of adjacent
+        # time values won't fit in 32 bits.
+        ntimes = 20
+        f = Dataset(self.file, 'r')
+        query_time = datetime(2037, 1, 3, 21, 12)
+        index = date2index(query_time, f.variables['time3'], select='nearest')
+        assert(index == 11)
+        f.close()
+
 if __name__ == '__main__':
     unittest.main()
